#ifndef ATTRIBUTEDATA_TREEVIEW_H
#define ATTRIBUTEDATA_TREEVIEW_H

#include <QTreeView>
#include <QHash>
#include "modelview/treeitem.h"
#include "database/attinfo.h"
#include "database/attdata.h"

class InvalidItemManager;
class TreeViewFilter;

class AttributeDataTreeView : public QTreeView
{
    Q_OBJECT
public:
    /* column date time is used in time machine where data is taken from hdb */
    enum Columns { ColumnDateTime = -1, ColumnAttr= 0, ColumnSetPoint,
                   ColumnRead, ColumnType, ColumnFormat, ColumnWritable };

    enum State { Init, InfoLoaded, ValuesLoaded };

    enum Mode  { Snap = 0, Hdb = 1, User = 100 };

    AttributeDataTreeView(QWidget *parent);

    QList<int> im1ValAttributes() const;
    QList<int> im2ValAttributes() const;
    QList<int> sc1ValAttributes() const;
    QList<int> sc2ValAttributes() const;
    QList<int> sp1ValAttributes() const;
    QList<int> sp2ValAttributes() const;
    QList<int> sc1ValStringAttributes() const;
    QList<int> sc2ValStringAttributes() const;

    State state() const { return m_state; }

    TreeViewFilter *treeViewFilter() const;

    int currentSnapId() const { return m_currentSnapId; }

    int currentContextId() const { return m_currentContextId; }

    virtual int getColumnIndex(Columns c) const;

    virtual int getRestoreColumnIndex() const;

    virtual QList<QVariant> getHeaderData();

    virtual void init();

    virtual void prepareForAttributeValuesQuery();

    virtual Mode getMode() const;

    QStringList getAttributeList();

public slots:

    virtual void update(const QList<AttData>&, AttInfo::Type);

    virtual void update(const QList<AttInfo>&);

    void setSnapId(int id) { m_currentSnapId = id; }

    void setContextId(int id) { m_currentContextId = id; }

    void launchHelperApplication();

    void selectByDevice(const QStringList &devList);

    void clear();

    void selectAll();

    void filter(const QString&);

    void setFilterMode(int mode);

    void sortBySection(bool bySection);

    void switchSortMode(int columnClicked);

    void setSectionsList(const QStringList&);

    QHash<int, int>& idRowHash() { return m_idRowHash; }

signals:
    void snapSelectionChanged(int snapId);

    void error(const QString& origin, const QString& msg);

    void attributeInfoDisplayed();

     void attributeValuesDisplayed();

     void selectionRequest();

     void showFilterRequest();

     void invalidItems(int readOnly, int null);

     void allItemsValid(bool allValid);

     void filterRegexpValid(bool);

     void snapInfoTreeViewSorted(int snapId);

protected:

    void contextMenuEvent(QContextMenuEvent *);

    virtual void stateChanged(State st);


    AttInfo::Type m_currentLastExpectedType;

    InvalidItemManager *m_invalidItemManager;

private:

    /* This hash assumes that the rows are never sorted and is aimed
     * at speeding up the row lookup in the update(const QList<AttValue>&)
     * method.
     */
    QHash<int, int> m_idRowHash;

    QList<int> m_im1val, m_im2val, m_sc1val, m_sc2val,
        m_sp1val, m_sp2val, m_sc1valStr, m_sc2valStr;

    int m_currentSnapId, m_currentContextId;

    QString m_devFromSource(const QString& src) const;

    /* this stores the current att info list, to avoid asking data
     * to the model in prepareForAttributeValuesQuery().
     * This should be faster and avoid some (rare) cases in which
     * an immediate selection of the snapshot right after the selection of
     * a context causes no values update on the snap info tree.
     */
    QList<AttInfo> m_currentAttInfoList;

    State m_state;


    QStringList m_orderedSectionsList;
};

#endif // SNAPINFOTREEVIEW_H
