#include "contextview.h"
#include "contextviewsettings.h"
#include "modelview/treeitem.h"
#include "modelview/itemmodel.h"
#include <macros.h>
#include <QVariant>
#include <QHeaderView>
#include <QtDebug>

ContextView::ContextView(QWidget *parent) : QTreeView(parent)
{
    mSelectedId = -1;
    setRootIsDecorated(false);
#if QT_VERSION >= 0x050000
    header()->setSectionResizeMode(QHeaderView::Interactive);
#else
    header()->setResizeMode(QHeaderView::Interactive);
#endif
    QList<QVariant> headerData;
    headerData << "Name" << "Time" << "Author" << "Reason" << "Description" << "Id";
    ItemModel *itemModel = new ItemModel(this, headerData);
    setModel(itemModel);

    /* manage context selection */
    connect(selectionModel(), SIGNAL(currentChanged(QModelIndex,QModelIndex)),
            this, SLOT(currentSelectionChanged(QModelIndex,QModelIndex)));
}

void ContextView::update(const QList<ContextData>& contexts)
{
    printf("updating context list\n");
    ContextViewSettings ctxViewSettings;
    QList<int> hiddenCtxList = ctxViewSettings.hiddenContexts();
    /* clear view */
    model()->removeRows(0, model()->rowCount());
    ItemModel *itemModel = static_cast<ItemModel *>(model());
    foreach(ContextData cd, contexts)
    {
        /* final false parameter not to emit dataChanged at every setData */
        if(!hiddenCtxList.contains(cd.id))
        {
            itemModel->insertRows(model()->rowCount(), 1);
            itemModel->setData(model()->index(model()->rowCount() - 1, 0), cd.name, Qt::DisplayRole, false);
            itemModel->setData(model()->index(model()->rowCount() - 1, 1), cd.time, Qt::DisplayRole, false);
            itemModel->setData(model()->index(model()->rowCount() - 1, 2), cd.author, Qt::DisplayRole, false);
            itemModel->setData(model()->index(model()->rowCount() - 1, 3), cd.reason, Qt::DisplayRole, false);
            itemModel->setData(model()->index(model()->rowCount() - 1, 4), cd.description, Qt::DisplayRole, false);
            itemModel->setData(model()->index(model()->rowCount() - 1, 5), cd.id, Qt::DisplayRole, false);
        }
    }
    /* trigger an update from the top left to the bottom right index */
    itemModel->update(itemModel->index(0, 0), itemModel->index(itemModel->rowCount() - 1, CtxId));
    header()->resizeSections(QHeaderView::ResizeToContents);
    if(mSelectedId > -1)
        selectId(mSelectedId);
}

void ContextView::currentSelectionChanged(const QModelIndex& current,const QModelIndex& previous)
{
    Q_UNUSED(previous);
    qDebug() << __FUNCTION__ << current << previous;
    if(current.isValid())
    {
        int row = current.row();
        int ctxId = model()->data(model()->index(row, model()->columnCount() - 1)).toInt();
        printf("itemActivated in context view: ctx id %d\n", ctxId);
        mSelectedId = ctxId;
        emit contextSelectionChanged(ctxId);
    }

}

void ContextView::selectId(int id)
{
    for(int i = 0; i < model()->rowCount(); i++)
    {
        if(model()->data(model()->index(i, CtxId)).toInt() == id)
        {
            QItemSelection is(model()->index(i, 0), model()->index(i, model()->columnCount() -1));
            printf("trying to select \e[1;32m%d\e[0m\n", id);
            qDebug() << __FUNCTION__ << is;
            selectionModel()->select(is, QItemSelectionModel::Select);
            emit contextSelectionChanged(id);
        }
    }
}

int ContextView::selectedContextId() const
{
    int contextId = -1;
    QItemSelection itemSelection = selectionModel()->selection();
    QModelIndexList idxList = itemSelection.indexes();
    if(idxList.size() == model()->columnCount())
        contextId = model()->data(model()->index(idxList.first().row(), CtxId), Qt::DisplayRole).toInt();
    printf("ContextView::selectedContextId() %d\n", contextId);
    return contextId;
}

