#include "contextviewsettings.h"
#include <QSettings>
#include <QFile>
#include <QTextStream>
#include <macros.h>
#include <QtDebug>
#include <QDir>

ContextViewSettings::ContextViewSettings()
{

}

QList<int> ContextViewSettings::hiddenContexts() const
{
    QList<int> hiddenContextList;
    QFile file(filename());
    if(!file.open(QIODevice::ReadOnly | QIODevice::Text))
    {
        pinfo("ContextViewSettings::hiddenContexts(): "
              "error opening file \"%s\" for reading: all contexts will be visible",
              qstoc(filename()));
    }
    else
    {
        bool inSection = false;
        QTextStream in(&file);
        while(!in.atEnd())
        {
            bool ok;
            QString line = in.readLine();

            if(line.endsWith('\n'))
                line.remove(line.length() - 1, 1); /* remove trailing '\n' */

            /* look for '[HIDDEN CONTEXTS]' directive in file */
            if(line.length() == 0 || line.startsWith('#'))
                ;
            else if(line == "[HIDDEN CONTEXTS]")
                inSection = true;
            else if(inSection)
            {
                int contextId = line.toInt(&ok);
                /* HIDDEN CONTEXTS block will contain the list of contexts
                 * one per line.
                 */
                if(ok)
                    hiddenContextList << contextId;
                else /* not a comment nor a white line: context list ends */
                    inSection = false;
            }
        }
        file.close();
    }
    qDebug() << __FUNCTION__ << "detected hidden contexts: " << hiddenContextList;
    return hiddenContextList;
}

QString ContextViewSettings::filename() const
{
    return filepath() + "context.conf";
}

QString ContextViewSettings::filepath() const
{
    QSettings s;
    QString path = s.value("SNAP_SELECTION_FILENAME",
        QDir::homePath() + "/etc/saverestore/").toString();
    if(!path.endsWith('/'))
        path += '/';
    return path;
}

