#include "addsnapshotquery.h"
#include <QDateTime>
#include <QSqlQuery>
#include <QVariant>
#include <QSqlRecord>
#include <QSqlError>
#include <macros.h>

AddSnapshotQuery::AddSnapshotQuery(QObject *parent) :
    DbQueryInterface(parent)
{
    m_contextId = -1;
}

void AddSnapshotQuery::init()
{
    m_contextId = -1;
    m_errorMessage.clear();
    m_comment.clear();
}

void AddSnapshotQuery::notify(const QSqlQuery& constQuery, const int eventId)
{
    if(eventId == m_contextId)
    {
        QSqlError err = constQuery.lastError();
        if(err.type() != QSqlError::NoError)
        {
            m_errorMessage = err.text();
            emit error("Failed to register new snapshot into snapshot table", m_errorMessage);
        }
        QVariant lastInsertId = constQuery.lastInsertId();

        if(lastInsertId.isValid()) /* no `else': suppose this never fails */
            emit snapAdded(m_errorMessage.isEmpty(), lastInsertId.toInt());
    }
    else
        perr("AddSnapshotQuery::notify(): eventId does not match contextId");
}

void AddSnapshotQuery::execute(int id)
{
    DbQueryInterface::execute(id);
}

QString AddSnapshotQuery::query(int id) const
{
    QString q;
    if(m_contextId > 0)
    {
        q = QString("INSERT INTO snapshot (id_context,time,snap_comment) "
                    " VALUES (%1,'%2','%3')").arg(id).
                arg(QDateTime::currentDateTime().toString("yyyy-MM-dd hh:mm:ss")).arg(m_comment);
    }
    return q;
}
