#include "attinfo.h"

AttInfo::AttInfo()
{

}

QString AttInfo::writeType() const
{
    switch(writable)
    {
    case Tango::READ:
        return "R";
    case Tango::READ_WITH_WRITE:
        return "RWW";
    case Tango::READ_WRITE:
        return "RW";
    case Tango::WRITE:
        return "W";
    default:
        return "INVALID";
    }
}

QString AttInfo::dataFormat() const
{
    switch(format)
    {
    case Tango::SCALAR:
        return "SCALAR";
    case Tango::SPECTRUM:
        return "SPECTRUM";
    case Tango::IMAGE:
        return "IMAGE";
    default:
        return "FMT_UNKNOWN";
    }
}

QString AttInfo::attrDataType() const
{
    switch(dataType)
    {
    case Tango::DEV_VOID:
        return "void";
    case Tango::DEV_BOOLEAN:
        return "bool";
    case Tango::DEV_SHORT:
        return "short";
    case Tango::DEV_FLOAT:
        return "float";
    case Tango::DEV_LONG:
        return "long";
    case Tango::DEV_DOUBLE:
        return "double";
    case Tango::DEV_USHORT:
        return "ushort";
    case Tango::DEV_UCHAR:
        return "uchar";
    case Tango::DEV_ULONG:
        return "ulong";
    case Tango::DEV_ULONG64:
        return "ulong64";
    case Tango::DEV_STRING:
        return "string";
    case Tango::DEV_STATE:
        return "state";
    case Tango::DEV_ENCODED:
        return "encoded";
    default:
        return QString::number(dataType);
    }
}

AttInfo::Type AttInfo::getType(Tango::CmdArgType dt, Tango::AttrDataFormat fmt, Tango::AttrWriteType wri)
{
    if(fmt == Tango::SPECTRUM)
    {
        if(wri == Tango::READ_WRITE || wri == Tango::READ_WITH_WRITE)
            return sp2;
        else
            return sp1;
    }
    else if(fmt == Tango::IMAGE)
    {
        if(wri == Tango::READ_WRITE || wri == Tango::READ_WITH_WRITE)
            return Im2;
        else
            return Im1;
    }
    else if(fmt == Tango::SCALAR && dt == Tango::DEV_STRING)
    {
        /* strings: there are two specific tables, for scalar formats only.
     */
        if(wri == Tango::READ_WRITE  || wri == Tango::READ_WITH_WRITE)
            return scStr2;
        else
            return scStr1;
    }
    else if(fmt == Tango::SCALAR && dt < Tango::DEV_ENCODED)
    {
        if(wri == Tango::READ_WRITE || wri == Tango::READ_WITH_WRITE)
            return scNum2;
        else if(fmt == Tango::SCALAR)
            return scNum1;
    }
    return Invalid;
}

