#include "attinfoquery.h"
#include <QSqlQuery>
#include <QVariant>
#include <QSqlRecord>

AttInfoQuery::AttInfoQuery(QObject *parent) : DbQueryInterface(parent)
{
    mSortBySectionEnabled = false;
}

void AttInfoQuery::notify(const QSqlQuery& constQuery, const int eventId)
{
    QList<AttInfo> attInfoList;
    QSqlQuery q(constQuery);
    while(q.next() && q.record().count() == 5)
    {
        AttInfo ai;
        ai.name = q.value(0).toString();
        ai.id = q.value(1).toInt();
        ai.setAttrDataType((Tango::CmdArgType) q.value(2).toInt());
        ai.setDataFormat((Tango::AttrDataFormat) q.value(3).toInt());
        ai.setWriteType((Tango::AttrWriteType)q.value(4).toInt());
        attInfoList << ai;
    }
    emit listReady(attInfoList);
}

void AttInfoQuery::execute(int id)
{
    DbQueryInterface::execute(id);
}

void AttInfoQuery::setSortBySectionEnabled(bool al)
{
    mSortBySectionEnabled = al;
}

QString AttInfoQuery::query(int id) const
{
    QString q;
    if(id > 0 && !mSortBySectionEnabled)
    {
        q = QString("SELECT full_name,ID,data_type,data_format,writable FROM ast,list"
                    " WHERE ast.ID=list.id_att AND id_context=%1 ORDER BY ast.full_name").arg(id);
    }
    else if(id > 0)
    {
        q = QString("SELECT full_name,ID,data_type,data_format,writable"
                    " FROM ast,list,section_order"
                    " WHERE ast.ID=list.id_att AND section_order.section=ast.domain AND "
                    " id_context=%1 ORDER BY section_order.position,ast.full_name").arg(id);
    }
    return q;
}


