#include "attvaluequery.h"
#include <macros.h>
#include <QSqlQuery>
#include <QVariant>
#include <QSqlRecord>
#include <QtDebug>

AttValueQuery::AttValueQuery(QObject *parent) :
    DbQueryInterface(parent)
{

}

void AttValueQuery::notify(const QSqlQuery& constQuery, const int eventId)
{
    bool formatError;
    QList<AttData> attValueList;
    QSqlQuery q(constQuery);
    while(q.next())
    {
        formatError = false;
        AttData av;
        av.id = q.value(1).toInt();
        av.idSnap = q.value(0).toInt();
        av.type = (AttInfo::Type) eventId;
        switch(eventId)
        {
        case AttInfo::scNum1:
        case AttInfo::scStr1:
            av.value = q.value(2).toString();
            break;
        case AttInfo::scNum2:
        case AttInfo::scStr2:
            av.value = q.value(2).toString();
            av.setpoint = q.value(3).toString();
            break;
        case AttInfo::sp1:
            av.value = q.value(3).toString();
            break;
        case AttInfo::sp2:
            av.value = q.value(3).toString();
            av.setpoint = q.value(4).toString();
            break;
        /* Tango images */
        case AttInfo::Im1:
            av.value = q.value(4).toString();
            break;
        case AttInfo::Im2:
            av.value = q.value(4).toString();
            av.setpoint = q.value(5).toString();
            break;
        default:
            formatError = true;
            perr("AttValueQuery::notify(): table type %d unsupported", eventId);
        }
        if(!formatError)
            attValueList << av;
    }
    /* emit even if list ready, the AttInfo::Type is used to detect the last
     * query notification by SnapInfoTreeView.
     */
    emit listReady(attValueList, (AttInfo::Type) eventId);
}

void AttValueQuery::execute(int id)
{
    DbQueryInterface::execute(id);
}

QString AttValueQuery::query(int id) const
{
    QString q;
    if(id > 0)
    {
        /* Im1 = 1, Im2, scNum1, scNum2, scStr1, scStr2, sp1, sp2 */
        switch(id)
        {
        case AttInfo::scNum1:
            q = QString("SELECT * FROM t_sc_num_1val WHERE id_snap=%1").arg(m_snapId);
            break;
        case AttInfo::scNum2:
            q = QString("SELECT * FROM t_sc_num_2val WHERE id_snap=%1").arg(m_snapId);
            break;
        case AttInfo::scStr1:
            q = QString("SELECT * FROM t_sc_str_1val WHERE id_snap=%1").arg(m_snapId);
            break;
        case AttInfo::scStr2:
            q = QString("SELECT * FROM t_sc_str_2val WHERE id_snap=%1").arg(m_snapId);
            break;
        case AttInfo::sp1:
            q = QString("SELECT * FROM t_sp_1val WHERE id_snap=%1").arg(m_snapId);
            break;
        case AttInfo::sp2:
            q = QString("SELECT * FROM t_sp_2val WHERE id_snap=%1").arg(m_snapId);
            break;

        case AttInfo::Im1:
        case AttInfo::Im2:
                perr("AttValueQuery::query(): image 1 val unsupported");
                break;
        default:
            perr("AttValueQuery::query(): table type %d unsupported", id);
        }
    }
    return q;
}


