#include <QMetaType>
#include "snapdb.h"
#include "settings/settings.h"
#include "database/snapdata.h"
#include "database/contextdata.h"
#include "database/attinfo.h"
#include "database/ctxlistquery.h"
#include "database/snaplistquery.h"
#include "database/attinfoquery.h"
#include "database/attvaluequery.h"
#include "database/sectionorderquery.h"

SnapDb::SnapDb(QObject *parent)
    : THdbBase(parent)
{
    qRegisterMetaType<SnapData>("SnapData");
    qRegisterMetaType<ContextData>("ContextData");
    qRegisterMetaType<AttInfo>("AttInfo");

    registerDbQuery("ctxListQuery", new CtxListQuery(this));
    registerDbQuery("snapListQuery", new SnapListQuery(this));
    registerDbQuery("attInfoQuery", new AttInfoQuery(this));
    registerDbQuery("attValueQuery", new AttValueQuery(this));
    registerDbQuery("sectionOrderQuery", new SectionOrderQuery(this));

    connect(dbQueryInterface("ctxListQuery"), SIGNAL(listReady(const QList<ContextData>&)),
            this, SIGNAL(contextListReady(const QList<ContextData>&)));

    connect(dbQueryInterface("snapListQuery"), SIGNAL(listReady(const QList<SnapData>&)),
            this, SIGNAL(snapListReady(const QList<SnapData>&)));

    connect(dbQueryInterface("attInfoQuery"), SIGNAL(listReady(const QList<AttInfo>&)),
            this, SIGNAL(attInfoListReady(const QList<AttInfo>&)));

    connect(dbQueryInterface("attValueQuery"), SIGNAL(listReady(const QList<AttData>&, AttInfo::Type)),
            this, SIGNAL(attValueListReady(const QList<AttData>&, AttInfo::Type)));

    connect(dbQueryInterface("sectionOrderQuery"), SIGNAL(listReady(const QStringList &)),
            this, SIGNAL(orderedSectionsReady(const QStringList &)));
}

void SnapDb::getContextList()
{
    dbQueryInterface("ctxListQuery")->execute();
}

void SnapDb::getSnapshotList(int contextId)
{
    dbQueryInterface("snapListQuery")->execute(contextId);
}

void SnapDb::getOrderedSectionsList()
{
    dbQueryInterface("sectionOrderQuery")->execute();
}

void SnapDb::getAttInfoList(int contextId)
{
    AttInfoQuery *attInfoQuery = static_cast<AttInfoQuery *>(dbQueryInterface("attInfoQuery"));
    Settings s;
    attInfoQuery->setSortBySectionEnabled(s.sectionSortingForContext(contextId));
    attInfoQuery->execute(contextId);
}

void SnapDb::getAttValueList(int snapId, int tableType)
{
    AttValueQuery *attValueQuery = static_cast<AttValueQuery *>(dbQueryInterface("attValueQuery"));
    attValueQuery->setSnapId(snapId);
    attValueQuery->execute(tableType);
}

