#ifndef DIFFTREEVIEW_H
#define DIFFTREEVIEW_H

#include <QTreeView>
#include <QVariant>
#include <QList>
#include "database/attinfo.h"
#include "database/attdata.h"

class SnapDb;
class QModelIndex;

class DiffAtt
{
public:
    int id;
    QString source;
};

class DiffTreeView : public QTreeView
{
    Q_OBJECT
public:
    explicit DiffTreeView(QWidget *parent = 0);

    virtual ~DiffTreeView();

    void setSnapshotIds(const QList<QVariant>& snaplist);

    QList<int> snapshotIds() const { return m_snapIdColumnHash.keys(); }

    QList<int> im1ValAttributes() const;
    QList<int> im2ValAttributes() const;
    QList<int> sc1ValAttributes() const;
    QList<int> sc2ValAttributes() const;
    QList<int> sp1ValAttributes() const;
    QList<int> sp2ValAttributes() const;
    QList<int> sc1ValStringAttributes() const;
    QList<int> sc2ValStringAttributes() const;

    int selectedSnapshotId() const;

    QList<DiffAtt> attributeList() const;

signals:
    void attInfoReady();

    void selectionChanged(int snapId);

    void validSnapSelection(bool valid);


public slots:
    void update(const QList<AttInfo>&);

    void update(const QList<AttData>&, AttInfo::Type);

    void liveDataAvailable(int attId, const QString& value, int quality, const QString& message);

    void updateLiveColumn();

    void liveViewEnabled(bool en);

    void percentChanged(double perc) { mPercent = perc; }

     void setHighlightDiffEnabled(bool en) { mHighlightDiffEnabled = en; }

     void setNoHighlightIfBelowThresholdEnabled(bool en) { mNoHighlightIfBelow = en; }

     void setNoHightlightThresh(const QString& threshAsStr);

      void setFilterOutNotHighlighted(bool f) { mFilterOutNotHighlighted = f; }

private slots:
    void currentSelectionChanged(const QModelIndex&, const QModelIndex&);

    void selectColumn(int);

private:

    QString shortenedValue(const QString& val) const;

    QList<int> m_im1val, m_im2val, m_sc1val, m_sc2val,
        m_sp1val, m_sp2val, m_sc1valStr, m_sc2valStr;

    int m_currentSnapId;

    QMultiHash<AttInfo::Type, int> m_attTypeInfoHash;
    QHash<int, int> m_idRowHash;
    QHash<int, int> m_snapIdColumnHash;

    double mPercent, mNoHighlightThresh;
    bool mHighlightDiffEnabled, mNoHighlightIfBelow, mFilterOutNotHighlighted;

};

#endif // DIFFTREEVIEW_H
