#include "invaliditemmanager.h"
#include "attributedatatreeview.h"
#include "timemachineattributedatatreeview.h"
#include "itemmodel.h"
#include <hdbxmacros.h>
#include <elettracolors.h>
#include <QAbstractItemModel>
#include <QAbstractItemView>
#include <QPaintEvent>
#include <QPainter>
#include <QString>

InvalidItemManager::InvalidItemManager(AttributeDataTreeView *view)
{
    m_view = view;
    m_nullCnt = 0;
    m_invalidWritableCnt = 0;
}

bool InvalidItemManager::findInvalidItems()
{
    m_nullCnt = 0;
    m_invalidWritableCnt = 0;
    ItemModel *m = static_cast<ItemModel *>(m_view->model());
    m->blockSignals(true);
    m_view->selectionModel()->blockSignals(true);

    Tango::AttrWriteType wri;
    int column = m_view->getRestoreColumnIndex();
    int columnWritable = m_view->getColumnIndex(AttributeDataTreeView::ColumnWritable);

    for(int i = 0; i < m->rowCount(); i++)
    {
        QString value = m->data(m->index(i, column), Qt::DisplayRole).toString();
        if(value == "NULL")
        {
            m_nullCnt++;
            m->setData(m->index(i, column), KRED, Qt::DecorationRole);
            printf("\e[1;31mnull value for %s: count %d\e[0m\n",
                   m->data(m->index(i, 0), Qt::DisplayRole).toString().toStdString().c_str(), m_nullCnt);
        }
        else if(m_view->getMode() == AttributeDataTreeView::Snap) /* needs to be RW or WO */
        {
            int writable = m->data(m->index(i, columnWritable), Qt::UserRole).toInt();
            wri = static_cast<Tango::AttrWriteType>(writable);
            if(wri == Tango::READ)
            {
                m_invalidWritableCnt++;
                m->setData(m->index(i, columnWritable), KDARKYELLOW, Qt::DecorationRole);
            }
        }
        else if(m_view->getMode() == AttributeDataTreeView::Hdb) /* needs to be RW or WO */
        {
            int writable = m->data(m->index(i, columnWritable), Qt::UserRole).toInt();
            wri = static_cast<Tango::AttrWriteType>(writable);
            if(wri == Tango::WRITE) /* write only */
            {
                m_invalidWritableCnt++;
                m->setData(m->index(i, columnWritable), KDARKYELLOW, Qt::DecorationRole);
            }
        }

        if(wri != Tango::READ)
          m->setData(m->index(i, columnWritable), QVariant(), Qt::DecorationRole);
        if(value != "NULL")
            m->setData(m->index(i, column), QVariant(), Qt::DecorationRole);
    }

    m->blockSignals(false);
    m_view->selectionModel()->blockSignals(false);
    printf("\e[0;31minvalid wri %d null %d\e[0m\n", m_invalidWritableCnt, m_nullCnt);
    return m_invalidWritableCnt > 0 || m_nullCnt > 0;
}

