#include "errorview.h"
#include <macros.h>
#include <QHeaderView>
#include <QDateTime>
#include "itemmodel.h"
#include <QTime>

ErrorView::ErrorView(QWidget *parent) :
    QTreeView(parent)
{

#if QT_VERSION >= 0x050000
    header()->setSectionResizeMode(QHeaderView::Interactive);
#else
    header()->setResizeMode(QHeaderView::Interactive);
#endif
    QList<QVariant> headerData;
    headerData << "Origin" << "Messsage" << "Date" << "Time";
    ItemModel *itemModel = new ItemModel(this, headerData);
    setModel(itemModel);

    /* manage context selection */
    connect(this, SIGNAL(pressed(QModelIndex)), this, SLOT(itemActivated(QModelIndex)));
}

QString ErrorView::chopMsg(const QString& original) const
{
   QString chopped = original;
   if(original.length() > 20)
   {
      chopped.chop(chopped.size() - 20);
      chopped += "...";
   }
   return chopped;
}

void ErrorView::addError(const QString& origin, const QString& msg)
{
     ItemModel *itemModel = static_cast<ItemModel *>(model());
     itemModel->insertRow(itemModel->rowCount());
//     /* origin and message */
     itemModel->setData(itemModel->index(itemModel->rowCount() -1, ColOrigin),
                        origin, Qt::DisplayRole, false);
     itemModel->setData(itemModel->index(itemModel->rowCount() -1, ColMessage),
                        chopMsg(msg), Qt::DisplayRole, false);
 
//     /* tooltip for message */
     itemModel->setData(itemModel->index(itemModel->rowCount() -1, ColOrigin),
			origin, Qt::ToolTipRole, false);
     itemModel->setData(itemModel->index(itemModel->rowCount() -1, ColMessage),
			tooltipToHtml(msg), Qt::ToolTipRole, false);

    QDateTime dt = QDateTime::currentDateTime();
    itemModel->setData(itemModel->index(itemModel->rowCount() -1, ColDate),
                     dt.date().toString("yyyy-MM-dd"), Qt::DisplayRole, false);
    itemModel->setData(itemModel->index(itemModel->rowCount() -1, ColTime),
                     dt.time().toString("hh:mm:ss"), Qt::DisplayRole, false);

 //  itemModel->update(itemModel->index(0, 0), itemModel->index(itemModel->rowCount() - 1, ColTime));

 //   header()->resizeSections(QHeaderView::ResizeToContents);

    if(!isVisible())
        emit visible(true);
}

void ErrorView::clear()
{
    model()->removeRows(0, model()->rowCount());
}

void ErrorView::itemActivated(const QModelIndex& )
{

}

QString ErrorView::tooltipToHtml(const QString& txtTooltip) const
{
    QString html = "<p>" + txtTooltip + "</p>";
    return html;
}


