#include "treeitem.h"
#include <macros.h>

TreeItem::TreeItem(const QList<QVariant> &data, TreeItem *parent)
{
    m_parentItem = parent;
    for(int i = 0; i < data.size(); i++)
        m_itemData.insert(i, data.at(i));
}

TreeItem::~TreeItem()
{
  //  printf("\e[1;31mTreeItem::~TreeItem(): \e[0m calling qDeleteAll\n");
    qDeleteAll(m_childItems);
    m_childItems.clear();
    m_itemData.clear();
    m_userData.clear();
    m_toolTipData.clear();
    m_decorationData.clear();
    m_backgroundData.clear();
    m_foregroundData.clear();
}

void TreeItem::removeChildren(int from, int count)
{
    int end;
    from + count < childCount() ? end = from + count : end = childCount();
    end--;

//    printf("data size of item %d\n", m_itemData.size());
//    if(end == from)
//        printf("\e[1;35m item %s has no children to remove\e[0m\n", qstoc(m_itemData.value(0).toString()));
//    else
//        printf("\e[1;32m item  has %d children to remove %s\e[0m\n", end - from, qstoc(m_itemData.value(0).toString()));
    for(int i = end; i >= from; i--)
    {
        TreeItem *item = m_childItems.at(i);
        delete item;
        m_childItems.removeAt(i);
    }
}

void TreeItem::appendChild(TreeItem *item)
{
    m_childItems.append(item);
}

TreeItem *TreeItem::child(int row)
{
    return m_childItems.value(row);
}

int TreeItem::childCount() const
{
    return m_childItems.count();
}

int TreeItem::row() const
{
    if (m_parentItem)
        return m_parentItem->m_childItems.indexOf(const_cast<TreeItem*>(this));

    return 0;
}

int TreeItem::columnCount() const
{
    return m_itemData.count();
}

QVariant TreeItem::data(int column, Qt::ItemDataRole role) const
{
    if(column < m_itemData.size())
    {
        switch(role)
        {
        case Qt::DisplayRole:
            return m_itemData.value(column);
        case Qt::ToolTipRole:
            return m_toolTipData.value(column);
        case Qt::UserRole:
            return m_userData.value(column);
        case Qt::DecorationRole:
            return m_decorationData.value(column);
        case Qt::ForegroundRole:
            return m_foregroundData.value(column);
        case Qt::BackgroundRole:
            return m_backgroundData.value(column);
        case Qt::FontRole:
            return m_fontData.value(column);
        default:
            perr("TreeItem::data(): unsupported role %d", role);
            break;
        }
    }
    return QVariant();
}

bool TreeItem::setData(const QVariant& val, int column, Qt::ItemDataRole role)
{
    if(column >= m_itemData.count())
        return false;
    switch(role)
    {
    case Qt::DisplayRole:
        m_itemData[column] = val;
        return true;
    case Qt::ToolTipRole:
        m_toolTipData[column] = val;
        return true;
    case Qt::UserRole:
        m_userData[column] = val;
        return true;
    case Qt::DecorationRole:
        m_decorationData[column] = val;
        return true;
    case Qt::ForegroundRole:
        m_foregroundData[column] = val;
        return true;
    case Qt::BackgroundRole:
        m_backgroundData[column] = val;
        return true;
    case Qt::FontRole:
        m_fontData[column] = val;
        return true;
    default:
        perr("TreeItem::data(): unsupported role %d", role);
        break;
    }
    return false;
}

TreeItem *TreeItem::parent()
{
    return m_parentItem;
}


