#include "treeviewfilter.h"
#include <macros.h>

TreeViewFilter::TreeViewFilter(QTreeView *view) : QObject(view)
{
    m_treeView = view;
    m_filterMode = Normal;
}

void TreeViewFilter::filter(const QString& text, int column)
{
    QAbstractItemModel *m = m_treeView->model();
    m_currentText = text;
    if(column < m->columnCount())
    {
        bool contains;
        QRegExp re;
        if(m_filterMode > Normal )
            re.setPattern(text);

        if(m_filterMode == WildCard)
            re.setPatternSyntax(QRegExp::Wildcard);

        emit regexpValid((!re.pattern().isEmpty() && re.isValid()) || m_filterMode == Normal);

        for(int i = 0; i < m->rowCount(); i++)
        {
            QString itemText = m->data(m->index(i, column)).toString();
            if(!re.pattern().isEmpty()) /* FilterMode is RegExp or WildCard */
                contains = itemText.contains(re);
            else
                contains = itemText.contains(text, Qt::CaseInsensitive);
            m_treeView->setRowHidden(i, QModelIndex(), !contains);
        }
    }
}

void TreeViewFilter::setFilterMode(FilterMode mode)
{
    m_filterMode = mode;
    /* update the view */
    filter(m_currentText, 0);
}
