#ifndef RESTORE_H
#define RESTORE_H

#include <QObject>
#include <QHash>
#include <tango.h>

class AttributeDataTreeView;
class Logger;

class Restore : public QObject
{
    Q_OBJECT
public:
    explicit Restore(QObject *parent, Logger *logger);

    unsigned writeErrorCount() const { return m_tangoErrors.size(); }

    QHash<QString, QString> tangoErrors() const { return m_tangoErrors; }

signals:
    void tangoWriteError(const QString& source, const QString& errorMsg);

    void finished();

    void progress(int step, int total);

    void status(const QString&);

    void started();

public slots:
    void restore(AttributeDataTreeView *view);

    void init();

    void deleteWriters();

    void clearErrors();

protected:
    bool itemRestorable(const QString& setPoint, int type, int format, int writable,
                        QString& reason) const;
    void checkForFinished();

protected slots:
    void error(const QString& target, const QString& msg);
    void writerTargetExecuted();

private:

    QHash<QString, QString> m_tangoErrors;
    int m_step, m_total;
    Logger *mLogger;

};

#endif // RESTORE_H
