#include "writer.h"
#include <macros.h>
#include <QVariant>

Writer::~Writer()
{
    pdelete("deleting writer \"%s\"", qstoc(objectName()));
}

Writer::Writer(QObject *parent, const QString& objectName, int snapId) :
    QObject(parent), QTangoComProxyWriter(this)
{
    mSnapId = snapId;
    setObjectName(objectName);
    connect(qtangoComHandle(), SIGNAL(connectionOk(bool)), this, SLOT(slotConnectionOk(bool)));
    connect(qtangoComHandle(), SIGNAL(connectionErrorMessage(QString)), this, SLOT(slotConnectionErrorMessage(QString)));
    connect(qtangoComHandle(), SIGNAL(attributeAutoConfigured(const TangoConfigurationParameters*)),
            this, SLOT(attributeConfigured(const TangoConfigurationParameters*)));
}

QList<TVariant> Writer::execute(QVariant v)
{
    /* create a dummy TVariant list. It is unused */
    QList<TVariant> dummyRet;
    dummyRet << TVariant(objectName());
    m_variant = v;
    return dummyRet;
}

void Writer::attributeConfigured(const TangoConfigurationParameters *)
{
    if(m_variant.isValid())
    {
        pinfo("Writer::attributeConfigured: execute(%s)", qstoc(m_variant.toString()));
        QList<TVariant> retList = QTangoComProxyWriter::execute(m_variant);
        if(retList.size() != 1 || retList.first().executionFailed())
            emit executionFailed(targets(), "write failed: " + retList.first().message());
        else
            emit targetExecuted();
    }
}

void Writer::slotConnectionOk(bool ok)
{
    if(!ok)
    {
        perr("Writer::slotConnectionOk: connection error. Clearing targets");
        clearTargets();
    }
}

void Writer::slotConnectionErrorMessage(const QString &msg)
{
    m_errorMessage = msg;
    perr("Writer::slotConnectionErrorMessage: connection error.");
    emit connectionFailed(objectName(), m_errorMessage);
}


