#ifndef COLLECTOR_H
#define COLLECTOR_H

#include <QObject>
#include <QHash>
#include <QSet>
#include <QMultiMap>

#include "../modelview/itemmodel.h"
#include "../database/attdata.h"
#include "../log/logger.h"

class Logger;

class Collector : public QObject
{
    Q_OBJECT
public:
    explicit Collector(QObject *parent, Logger *logger);

    QMultiMap<AttInfo::Type, AttData>& attValues() { return m_attValues; }

    QHash<QString, QString> attErrors() const { return m_attErrors; }
    QHash<QString, QString> attMessages() const { return m_attMessages; }

    QList<AttInfo::Type> types() const;

    void deleteReaders();

    void setLogger(Logger *logger) { mLogger = logger; }


signals:
    void progress(int step, int total);
    void finished();

public slots:

    virtual void read(const QAbstractItemModel *model, const int snapId);

private slots:
    void readerConnectionOk(bool ok);
    void readerDataAvailable(bool ok);
    void slotConnectionErrorMessage(const QString&);

private:
    QMultiMap<AttInfo::Type, AttData> m_attValues;
    QHash<QString, QString> m_attErrors;
    QHash<QString, QString> m_attMessages;
    QSet<QString> m_remainingSources;
    unsigned m_totSources;

    /* initialize internal data structures */
    void init();

    Logger* mLogger;
};

#endif // COLLECTOR_H
