#ifndef SAVE_H
#define SAVE_H

#include <QObject>
#include <QHash>
#include "collector.h"
#include "../database/snapdb.h"
#include "../database/attinfo.h"

class SnapInsertQuery;
class Logger;

class Save : public QObject
{
    Q_OBJECT
public:
    explicit Save(QObject* parent, SnapDb *snapDb, Logger* logger);

    unsigned readErrorCount() const;
    QHash<QString, QString> attErrors() const;

    unsigned dbErrorCount() const { return m_dbErrorCnt; }

    void setModel(QAbstractItemModel *model) { m_model = model; }

signals:
    void readProgress(int stepNo, int totSteps);
    void dbWriteProgress(int stepNo, int totSteps);
    void dbWriteError(const QString& query, const QString& message);
    void finished(int contextId, int newSnapshotId);
    void finished(int contextId);
    void status(const QString& text);
    void started();

public slots:
    void save(const QString& comment);
    void setContextId(int id) { m_contextId = id; }


private slots:
    void collectorFinished();

    void dataQueryExecuted(const QString& q, bool success, const QString& errorMsg = QString());

    /* stores data into data tables */
    void saveData(bool ok, int snapId);

private:
    /* maps attribute id and query */
    QHash<AttInfo::Type, SnapInsertQuery *> m_activeQueries;
    unsigned m_queryCnt, m_dbErrorCnt, m_readErrorCnt;
    int m_contextId;
    SnapDb* m_snapDb;
    QAbstractItemModel *m_model;
};

#endif // SAVE_H
