#ifndef SAVERESTORE_H
#define SAVERESTORE_H

#include <QMainWindow>

namespace Ui {
    class SaveRestore;
}

class Logger;
class AttributeDataTreeView;

class SaveRestore : public QMainWindow
{
    Q_OBJECT

public:
    explicit SaveRestore(QWidget *parent = 0);
    ~SaveRestore();

    bool isTimeMachine() const;

public slots:
    void setTimeMachineEnabled(bool timeMachine);

signals:
    void selectionDialogShown();
    void selectionDialogClosed();


private slots:
    void save();
    void restore();
    void saveFinished(int contextId, int snapId);
    void restoreFinished();
    void showDiffDialog();
    void executeSelectionDialog();
    void selectionDialogClosed(int result);
    void executeConfigDialog();
    void executeSelectFromDiaryDialog();
    void progress(int s, int t);
    void extractionProgress(int s, int t);
    void restoreProgress(int s, int t);
    void extractFromHdb();

private:

    void mSetupSaveRestore();
    void mSetupTimeMachine();
    void mSetupCommon();
    void mDeleteTimeMachine();
    void mDeleteSaveRestore();
    void mSetupCommon2();

    AttributeDataTreeView *getAttributeDataTreeView() const;

    Ui::SaveRestore *ui;

    Logger *mLogger;

    /* Used to save the previously selected context id and restore the
     * snapshot information after switching mode on the AttributeDataTreeView.
     * Saved in setTimeMachineEnabled, restored in mSetupCommon2.
     */
    int mSelectedContextId;

};

#endif // SAVERESTORE_H
