#include "selection.h"
#include "../database/snapdb.h"
#include "../database/selectiontypequery.h"
#include "../database/deviceselectionquery.h"


Selection::Selection(QObject *parent, SnapDb* snapDb, int contextId,
                     const QItemSelection &currentSelection) :
    QObject(parent)
{
    m_snapDb = snapDb;
    m_contextId = contextId;
    m_savedItemSelection = currentSelection;
    SelectionTypeQuery *selectionTypeQuery = new SelectionTypeQuery(this);
    snapDb->registerDbQuery("selectionTypeQuery", selectionTypeQuery);
    connect(selectionTypeQuery, SIGNAL(selectionTypesAvailable(int,QStringList)),
            this, SLOT(slotTypesAvailable(int, QStringList)));

    DeviceSelectionQuery *deviceSelectionQuery = new DeviceSelectionQuery(this);
    snapDb->registerDbQuery("deviceSelectionQuery", deviceSelectionQuery);
    connect(deviceSelectionQuery, SIGNAL(deviceAvailable(QString,QString,QString)),
            this, SIGNAL(deviceReady(QString,QString,QString)));
    connect(deviceSelectionQuery, SIGNAL(dataLoaded()), this, SIGNAL(dbDataRetrieved()));
}

void Selection::getSelection()
{
    SelectionTypeQuery *selectionTypeQuery =
            static_cast<SelectionTypeQuery *>(m_snapDb->dbQueryInterface("selectionTypeQuery"));
    selectionTypeQuery->execute(m_contextId);
}

void Selection::slotTypesAvailable(int contextId, const QStringList& types)
{
    emit typesReady(types);
    DeviceSelectionQuery *deviceSelectionQuery = static_cast<DeviceSelectionQuery*>
            (m_snapDb->dbQueryInterface("deviceSelectionQuery"));
    deviceSelectionQuery->execute(contextId);
}
