#include "selectiondialog.h"
#include "tableitemdelegate.h"
#include "selectionsettings.h"
#include <QHeaderView>
#include <QCheckBox>
#include <macros.h>

SelectionDialog::~SelectionDialog()
{
    QList<Cell > selectedCells = ui.table->selectedCells();
    SelectionSettings selectionSettings;
    pinfo("SelectionDialog: saving selection settings for context %d", m_contextId);
    selectionSettings.save(m_contextId, selectedCells);
}

SelectionDialog::SelectionDialog(QWidget *parent, int contextId) :
    QDialog(parent)
{
    ui.setupUi(this);

    setAttribute(Qt::WA_DeleteOnClose, true);

    m_contextId = contextId;

    connect(ui.table, SIGNAL(deviceCellChanged(const QStringList&)),
            this, SLOT(deviceCellChanged(const QStringList&)));

    connect(ui.table, SIGNAL(deviceSelectionChanged(const QStringList&)),
            this, SIGNAL(selectionChanged(const QStringList&)));


}

void SelectionDialog::addDevice(const QString &section, const QString &type, const QString &dev)
{
    ui.table->addDevice(section, type, dev);
}

void SelectionDialog::addTypes(const QStringList& typeList)
{
    foreach(QString type, typeList)
        ui.table->addType(type);
}

void SelectionDialog::deviceCellChanged(const QStringList& list)
{
    ui.listWidget->clear();
    ui.listWidget->insertItems(0, list);
}

void SelectionDialog::dataLoadedFromDb()
{
    SelectionSettings selectionSettings;
    ui.table->select(selectionSettings.cellSelection(m_contextId));
    /* disable cells without associated devices */
    ui.table->disableCellsWithoutDevices();
}
