#include "selectionsettings.h"
#include <QFile>
#include <QTextStream>
#include <QSettings>
#include <QDir>
#include <macros.h>
#include <QtDebug>

Cell::Cell()
{

}

SelectionSettings::SelectionSettings()
{

}

void SelectionSettings::save(int contextId, const QList<Cell >& cells)
{
    QFile file(filename(contextId));
    if(file.open(QIODevice::WriteOnly|QIODevice::Text))
    {
        QTextStream out(&file);
        foreach(Cell c, cells)
        {
            out << c.type << "," << c.section << "\n";
        }
        file.close();
    }
}

QList<Cell > SelectionSettings::cellSelection(int contextId) const
{
    QList<Cell > cells;
    QFile file(filename(contextId));
    printf("\e[1;32mcellSelection enters %s\e[0m\n", qstoc(file.fileName()));
    if(file.open(QIODevice::ReadOnly|QIODevice::Text))
    {
        QTextStream in(&file);
        while(!in.atEnd())
        {
            QString line = in.readLine();
            line.remove("\n");
            if(line.count(",") == 1)
            {
                QStringList spl = line.split(",", QString::SkipEmptyParts);
                if(spl.size() == 2)
                    cells << Cell(spl.first(), spl.last());
            }
        }
        file.close();
    }
    else
        perr("SelectionSettings: cellSelection: failed to open file \"%s\" for reading: %s",
             qstoc(file.fileName()), qstoc(file.errorString()));
    return cells;
}

QString SelectionSettings::filename(int contextId) const
{
    return filepath() + QString::number(contextId);
}

QString SelectionSettings::filepath() const
{
    QSettings s;
    QString path = s.value("SNAP_CELLS_SELECTION_FILENAME",
        QDir::homePath() + "/etc/saverestore/cell_selection/").toString();
    if(!path.endsWith('/'))
        path += '/';
    return path;
}

