#include "settings.h"
#include <QSettings>
#include <QtDebug>

Settings::Settings()
{
}

bool  Settings::sectionSortingForContext(int contextId) const
{
    if(contextId < 0)
        return false;
    QSettings s;
    return s.value(QString("SECTION_SORTING_FOR_CONTEXT_%1_%2")
                   .arg(contextId).arg(s.value("DBHOST", "ken").toString()),
                   false).toBool();
}

void  Settings::setSectionSortingForContext(int contextId, bool enable)
{
    if(contextId < 0)
        return;

    QSettings s;
    s.setValue(QString("SECTION_SORTING_FOR_CONTEXT_%1_%2").
               arg(contextId).arg(s.value("DBHOST", "ken").toString()), enable);
}

QString Settings::diarySource() const
{
    QSettings s;
    return s.value("DIARY_SOURCE", "archiving/time_machine/fermi/diary").toString();
}

HdbXSettings *Settings::getHdbXSettings() const
{
    HdbXSettings *q = new HdbXSettings();
    QSettings s;
    q->set("dbhost", s.value("DBHOST_HDB", "ken").toString().toStdString().c_str());
    q->set("dbname", s.value("DBNAME_HDB", "hdb").toString().toStdString().c_str());
    q->set("dbuser", s.value("DBUSER_HDB", "hdbbrowser").toString().toStdString().c_str());
    q->set("dbpass", s.value("DBPASSWORD_HDB", "hdbbrowser").toString().toStdString().c_str());
    q->set("dbport", s.value("DBPORT_HDB", 3306).toString().toStdString().c_str());
    return q;
}

DbConnectionParams Settings::dbConnectionParams(bool snap) const
{
    DbConnectionParams dbParams;
    QSettings s;
    if(snap)
    {
        dbParams.dbHost = s.value("DBHOST", "ken").toString();
        dbParams.dbPort = s.value("DBPORT", 3306).toInt();
        dbParams.dbName = s.value("DBNAME", "snap").toString();
        dbParams.dbUser = s.value("DBUSER", "snaparchiver").toString();
        dbParams.dbPassword = s.value("DBPASSWORD", "snaparchiver").toString();
        qDebug() << __FUNCTION__ << dbParams.dbHost <<  dbParams.dbName;
    }
    else
    {
        dbParams.dbHost = s.value("DBHOST_SNAP_TIME_MACHINE", "ken").toString();
        dbParams.dbPort = s.value("DBPORT_SNAP_TIME_MACHINE", 3306).toInt();
        dbParams.dbName = s.value("DBNAME_SNAP_TIME_MACHINE", "snap").toString();
        dbParams.dbUser = s.value("DBUSER_SNAP_TIME_MACHINE", "snaparchiver").toString();
        dbParams.dbPassword = s.value("DBPASSWORD_SNAP_TIME_MACHINE", "snaparchiver").toString();
    }
    return dbParams;
}
