#include "snapinfotreeviewsorter.h"
#include "itemmodel.h"
#include <QHeaderView>
#include <QtDebug>

RowData::RowData(QAbstractItemModel *model, int  row)
{
    section = model->data(model->index(row, AttributeDataTreeView::ColumnAttr), Qt::DisplayRole)
            .toString().section('/', 0, 0);

    /* save the whole data of the row */
    for(int col = 0; col < model->columnCount(); col++)
    {
        displayData << model->data(model->index(row, col), Qt::DisplayRole).toString();
        toolTipData << model->data(model->index(row, col), Qt::ToolTipRole).toString();
        userData << model->data(model->index(row, col), Qt::UserRole).toString();
    }
}

SnapInfoTreeViewSorter::SnapInfoTreeViewSorter(AttributeDataTreeView *view,
                                               bool bySection,
                                               const QStringList &sections)
{
    QList<RowData *> rows;
    ItemModel *model = static_cast<ItemModel *>(view->model());

    /* fill in the list of row data with the data contained in each row of the model */
    for(int i = 0; i < model->rowCount(); i++)
    {
        RowData *rowData = new RowData(model, i);
        rows.append(rowData);
    }
    if(bySection && sections.size() > 0)
    {
        mSort(sections, rows, bySection);
    }
    else
    {
        QStringList familyList;
        foreach(RowData *rd, rows)
            familyList.append(rd->displayData.at(AttributeDataTreeView::ColumnAttr));
        /* sort alphabetically */
        qSort(familyList.begin(), familyList.end());
        /* then invoke the private mSort() method on the alphabetically ordered list
         * and passing rows as parameter.
         */
        mSort(familyList, rows, bySection);
    }

    /* now we have the rows list ordered according to bySection value.
     * We can proceed to the actual update of the tree view
     */
    view->idRowHash().clear();
    model->removeRows(0, model->rowCount());
    model->insertRows(0, rows.count());
    int row = 0;

    foreach(RowData *rd, rows)
    {
        /* Must rebuild the idRow hash to ensure that subsequent calls to
         * SnapInfoTreeView::update(const QList<AttValue>&, AttInfo::Type);
         * are correctly managed.
         */
        view->idRowHash().insert(rd->userData.at(AttributeDataTreeView::ColumnAttr).toInt(), row);

        for(int col = 0; col < model->columnCount(); col++)
        {
            model->setData(model->index(row, col), rd->displayData.at(col), Qt::DisplayRole, false);
            model->setData(model->index(row, col), rd->userData.at(col), Qt::UserRole, false);
            model->setData(model->index(row, col), rd->toolTipData.at(col), Qt::ToolTipRole, false);
        }
        row++;
    }

    /* trigger an update from the top left to the bottom right index */
    model->update(model->index(0, 0), model->index(model->rowCount(), model->columnCount()));
    view->header()->resizeSections(QHeaderView::ResizeToContents);

    /* delete temporary data used to swap items */
    foreach(RowData *rd, rows)
        delete rd;
}

/* orders rows in place according to the orderedSections string list
 */
void SnapInfoTreeViewSorter::mSort(const QStringList& orderedStringList, QList<RowData *>& rows, bool bySection)
{
    int lastBroughtUpIndex = 0;
    foreach(QString section, orderedStringList)
    {
        for(int i = lastBroughtUpIndex; i < rows.size(); i++)
        {
            /* get the section from the first column (Attribute) referring to the device family */
            QString currentStr;
            bySection ? currentStr = rows.at(i)->section :
                    currentStr = rows.at(i)->displayData.at(AttributeDataTreeView::ColumnAttr);

            if(!currentStr.compare(section, Qt::CaseInsensitive))
            {
                RowData *tempPtr = rows.at(i);
                rows[i] = rows[lastBroughtUpIndex];
                rows[lastBroughtUpIndex] = tempPtr;
                lastBroughtUpIndex++;
            }
        }
    }
}
