#ifndef EXTRACTION_H
#define EXTRACTION_H

#include <QThread>
#include <QList>
#include "attinfo.h"
#include "attdata.h"
#include <QVector>
#include <QStringList>

#include <xvariant.h>
#include <hdbextractorlistener.h>

class ExtractorPrivate;
class QEvent;
class HdbXSettings;

class Extractor : public QThread, public HdbExtractorListener
{
    Q_OBJECT
public:
    explicit Extractor(QObject *parent = 0);

    virtual ~Extractor();

      void getData(const QDateTime& startDt, const QDateTime &stopDt, const QStringList &attributes);

      void getData(const QDateTime& dateTime, const QStringList& attributes);

      QStringList getErrors();

      bool hasError();

      void setProgressUpdateGranularity(int steps);

      void clearErrors();

      void cancel();

private slots:
      void threadFinished();

protected:

      bool event(QEvent *e);

      virtual void onSourceProgressUpdate(const char*, double);

      virtual void onExtractionFinished(int, double);

      virtual void onSourceExtractionFinished(const char* source, int totalRows, double elapsed);

      void run();

      void publishResult();

signals:
    void attValueListReady(const QList<AttData>& values, AttInfo::Type type);

    void progress(int step, int total);

    void onError(const QString& origin, const QString& error);

private:

    void mSetParamsFromHdbXSettings(HdbXSettings *q);

    AttInfo::Type mGetAttInfoType(XVariant::DataFormat, XVariant::DataType, XVariant::Writable);

    ExtractorPrivate *d_ptr;
};

#endif // EXTRACTION_H
