#include "timemachineinfoviewdbhelper.h"
#include "timemachineinfotreeview.h"
#include "extraction/extractor.h"

TimeMachineInfoViewDbHelper::TimeMachineInfoViewDbHelper(SnapInfoTreeView *sitw, Extractor *extractor) :
    QObject(sitw)
{
    m_extractor = extractor;
    m_treeView = sitw;

    connect(snapDb, SIGNAL(attValueListReady(const QList<AttValue>&, AttInfo::Type)),
            this, SIGNAL(attValueListReady(const QList<AttValue>&, AttInfo::Type)));
}

void TimeMachineInfoViewDbHelper::getData()
{
    QDateTime dt = ui->dte->dateTime();

    printf("\e[0;33mTimeMachineInfoViewDbHelper::getData id %d\e[0m\n", snapId);
    /* gather all sets of attribute ids for each category: sc1val, sc2val, sp1val, sp2val..
     * and so on.
     * Then, for each non empty set, post a query to the database, according to the type
     * of the category.
     * When the results come back, the QSqlQuery result will contain the attribute ID in one
     * of its records, so it is straightforward to associate the values to the attributes.
        QList<int> im1ValAttributes() const;
        QList<int> im2ValAttributes() const;
        QList<int> sc1ValAttributes() const;
        QList<int> sc2ValAttributes() const;
        QList<int> sp1ValAttributes() const;
        QList<int> sp2ValAttributes() const;
        QList<int> sc1ValStringAttributes() const;
        QList<int> sc2ValStringAttributes() const;
     */
    /* divide attributes in snap info tree view into categories depending by format,
     * writable and type characteristics.
     * Creates attid/model index hash table
     */
    m_treeView->prepareForAttributeValuesQuery();


    /* get the values in the same order as the Types appear in the AttInfo enum:
     * enum Type { Im1 = 1, Im2, scNum1, scNum2, scStr1, scStr2, sp1, sp2, Invalid };
     */

    /* IMAGE 1 and 2 VAL */
    if(m_treeView->im1ValAttributes().size() > 0)
        m_extractor->getData(dt, m_treeView->im1ValAttributes());
        m_snapDb->getAttValueList(snapId, AttInfo::Im1);

    if(m_treeView->im2ValAttributes().size() > 0)
        m_snapDb->getAttValueList(snapId, AttInfo::Im2);

    /* SCALAR NUMBERS */
    if(m_treeView->sc1ValAttributes().size() > 0)
        m_snapDb->getAttValueList(snapId, AttInfo::scNum1);

    if(m_treeView->sc2ValAttributes().size() > 0)
        m_snapDb->getAttValueList(snapId, AttInfo::scNum2);

    /* SCALAR STRINGS */
    if(m_treeView->sc1ValStringAttributes().size() > 0)
        m_snapDb->getAttValueList(snapId, AttInfo::scStr1);

    if(m_treeView->sc2ValStringAttributes().size() > 0)
        m_snapDb->getAttValueList(snapId, AttInfo::scStr2);

    /* SPECTRUM 1 and 2 VAL */
    if(m_treeView->sp1ValAttributes().size() > 0)
        m_snapDb->getAttValueList(snapId, AttInfo::sp1);

    if(m_treeView->sp2ValAttributes().size() > 0)
    {
        printf("\e[1;36mgetting sp 2 val attributes\e[0m\n");
        m_snapDb->getAttValueList(snapId, AttInfo::sp2);
    }
}
