#include "uihelper.h"
#include "attributedatatreeview.h"
#include "snapshotlistcontainer.h"
#include <macros.h>
#include <QWidget>
#include <QPushButton>
#include <QAbstractItemModel>
#include <QApplication>
#include <QTreeView>
#include <QLabel>
#include <QFrame>
#include <QSplitter>
#include <QGroupBox>
#include <QLineEdit>
#include <QStatusBar>
#include <QComboBox>
#include <QtDebug>
#include <QMainWindow>
#include <QStatusBar>
#include <QProgressBar>
#include "elettracolors.h"
#include "groupboxbackintime.h"

UiHelper::UiHelper(QWidget *mainWin) : QObject(mainWin)
{
    m_mainWin = mainWin;
}

void UiHelper::init()
{
    m_mainWin->findChild<QFrame *>("frameDeleteContext")->setVisible(false);
    m_mainWin->findChild<QPushButton *>("pbRestore")->setDisabled(true);
    /* null values push button and label: hide them */
    m_mainWin->findChild<QLabel *>("labelNullValues")->setHidden(true);
    m_mainWin->findChild<QPushButton *>("pbIKnow")->setHidden(true);

    setViewFilterVisible(false);
    setDiffButtonsEnabled(false);
}

void UiHelper::setViewFilterVisible(bool show)
{
    m_mainWin->findChild<QLineEdit *>("leFilter")->setVisible(show);
    m_mainWin->findChild<QLineEdit *>("leFilter")->setFocus();
    m_mainWin->findChild<QPushButton *>("pbClearFilter")->setVisible(show);
    m_mainWin->findChild<QPushButton *>("pbHideFilter")->setVisible(show);
    m_mainWin->findChild<QComboBox *>("cbFilterOptions")->setVisible(show);
}

bool UiHelper::filterVisible() const
{
    return m_mainWin->findChild<QLineEdit *>("leFilter")->isVisible();
}

QString UiHelper::filterString() const
{
    return m_mainWin->findChild<QLineEdit *>("leFilter")->text();
}

void UiHelper::expandContextView(bool expand)
{
    m_mainWin->findChild<QWidget *>("rightWidget")->setHidden(expand);
    m_mainWin->findChild<QSplitter *>("leftSplitter")->widget(1)->setHidden(expand);
}

void UiHelper::contextSelected()
{
    GroupBoxBackInTime *gbt = m_mainWin->findChild<GroupBoxBackInTime *>();
    if(gbt)
        gbt->setEnabled(true);
    expandContextView(false);
    emit contextViewCollapsed(false);
}

void UiHelper::snapSelected()
{
    bool filterDisabled = m_mainWin->findChild<QLineEdit *>("leFilter")->text().isEmpty();
    m_mainWin->findChild<QPushButton *>("pbRestore")->setDisabled(filterDisabled);
}

void UiHelper::optimizeSnapshotView (bool optimize)
{
    QSplitter *leftSplitter = m_mainWin->findChild<QSplitter *>("leftSplitter");
    int ctxHeight = m_mainWin->findChild<QWidget *>("widgetCtx")->height();
    int snapHeight = 0;
    if(optimize)
    {
        ctxHeight *= 0.5;
        snapHeight *= 1.5;
    }
    else
    {
        snapHeight *= 0.5;
        ctxHeight *= 1.5;
    }
    leftSplitter->setSizes(QList<int>() << ctxHeight << snapHeight);
}

void UiHelper::ensureErrorViewVisible()
{
    expandContextView(false);
    m_mainWin->findChild<QGroupBox *>("gbErrors")->setVisible(true);
    m_mainWin->findChild<QGroupBox *>("gbErrors")->setChecked(true);
}

void UiHelper::snapInfoViewFilterTextChanged(const QString& text)
{
    bool valuesInSnapInfoView = m_mainWin->findChild<AttributeDataTreeView *>()->state()
            == AttributeDataTreeView::ValuesLoaded;
    QPushButton *pbIKnow = m_mainWin->findChild<QPushButton *>("pbIKnow");
    /* if push button "Yes, I Know" is enabled and visible, then it means that there are
     * null or read only values in the current snap info view that have not been acknowledged
     * yet. So, in this case, we cannot enable the Restore push button.
     */
    if(!pbIKnow->isVisible() || !pbIKnow->isEnabled())
        m_mainWin->findChild<QPushButton *>("pbRestore")->setEnabled(text.isEmpty() && valuesInSnapInfoView);
}

void UiHelper::selectionDialogShown()
{
    m_mainWin->findChild<QWidget *>("leftWidget")->setDisabled(true);
}

void UiHelper::selectionDialogClosed()
{
    m_mainWin->findChild<QWidget *>("leftWidget")->setDisabled(false);
}

void UiHelper::contextSelectionChanged(int id)
{
    /* user cannot select items in the snap list view and in the snap info view */
    setDiffButtonsEnabled(false);
    m_mainWin->findChild<QPushButton *>("pbRestore")->setDisabled(true);
    /* null values push button and label: hide them */
    m_mainWin->findChild<QLabel *>("labelNullValues")->setHidden(true);
    m_mainWin->findChild<QPushButton *>("pbIKnow")->setHidden(true);
    contextSelected();
}

void UiHelper::snapSelectionChanged(int id)
{
    bool filterDisabled = m_mainWin->findChild<QLineEdit *>("leFilter")->text().isEmpty();
    setDiffButtonsEnabled(true);
    m_mainWin->findChild<QPushButton *>("pbRestore")->setEnabled(filterDisabled);
    m_mainWin->findChild<QGroupBox *>("groupBoxSnapshotDetails")->setTitle(
                QString("Snapshot %1 details").arg(id));
    QProgressBar * pb = m_mainWin->findChild<QStatusBar *>()->findChild<QProgressBar *>();
    pb->setMinimum(0);
    pb->setMaximum(0);
    pb->setVisible(true);
}

void UiHelper::attributeValuesReady()
{
    QProgressBar * pb = m_mainWin->findChild<QStatusBar *>()->findChild<QProgressBar *>();
    pb->setVisible(false);
}

void UiHelper::dbDataRetrievalStart()
{
    QApplication::setOverrideCursor(QCursor(Qt::WaitCursor));
    m_mainWin->findChild<QStatusBar *>()->showMessage("waiting for data from snapshot db...");
}

void UiHelper::dbDataRetrievalEnd(bool result)
{
    QApplication::restoreOverrideCursor();
    if(result)
        m_mainWin->findChild<QStatusBar *>()->showMessage("correctly retrieved data from snap db...");
    else
        m_mainWin->findChild<QStatusBar *>()->showMessage("error retrieving data from snap db...");
}

void UiHelper::allItemsValidInSnap(bool allValid)
{
    qDebug() << __FUNCTION__ << "ALL VALID " << allValid;
  //  setDiffButtonsEnabled(allValid);
    m_mainWin->findChild<QLabel *>("labelNullValues")->setHidden(allValid);
    /* disable restore button. It will be enabled again when the user presses the "I know " button */
    m_mainWin->findChild<QPushButton *>("pbRestore")->setEnabled(allValid);
    m_mainWin->findChild<QPushButton *>("pbIKnow")->setVisible(!allValid);
}

void UiHelper::invalidItemsInSnap(int incompatibleWritable, int nullItems)
{
    QString txt;
    if(incompatibleWritable > 0 && nullItems > 0)
        txt = QString("There are %1 incompatible write type and %2 null items").arg(incompatibleWritable).arg(nullItems);
    else if(incompatibleWritable > 0)
        txt = QString("There are %1 attributes with an incompatible write type that won't be restored.").arg(incompatibleWritable);
    else if(nullItems > 0)
        txt = QString("There are %1 attributes with NULL values that can't be restored.").arg(nullItems);
    /* tool tip */
    m_mainWin->findChild<QLabel *>("labelNullValues")->setToolTip("<p>Attributes with NULL values or READ ONLY"
           " cannot be restored. NULL values mean that a read error occurred while saving the snapshot.</p>"
           "<p>Attributes with NULL values must be restored manually. Right clicking on an item and "
           "choosing <em>helper application</em> may help.</p>"                                                       );
    /* text */
    m_mainWin->findChild<QLabel *>("labelNullValues")->setText(txt);
    /* disable restore button. It will be enabled again when the user presses the "I know " button */
    m_mainWin->findChild<QPushButton *>("pbRestore")->setEnabled(false);
    m_mainWin->findChild<QPushButton *>("pbIKnow")->setEnabled(true);
}

void UiHelper::iKnowClicked()
{
    m_mainWin->findChild<QPushButton *>("pbRestore")->setEnabled(true);
    m_mainWin->findChild<QPushButton *>("pbIKnow")->setEnabled(false);
}

void UiHelper::setFilterLineEditValid(bool valid)
{
    QLineEdit *filterLineEdit = m_mainWin->findChild<QLineEdit *>("leFilter");
    QPalette palette = filterLineEdit->palette();
    if(!valid)
        palette.setColor(QPalette::Base, KRED.lighter());
    else
        palette.setColor(QPalette::Base, Qt::white);
    filterLineEdit->setPalette(palette);
}

void UiHelper::setDiffButtonsEnabled(bool enable)
{
    SnapshotListContainer *slc = m_mainWin->findChild<SnapshotListContainer *>();
    if(slc)
         m_mainWin->findChild<SnapshotListContainer *>()->findChild<QPushButton *>("pbDiff")->setEnabled(enable);
    return m_mainWin->findChild<QPushButton *>("pbDiffTimeMachine")->setEnabled(enable);
}

void UiHelper::extractFromHdbStarted()
{
    QApplication::setOverrideCursor(QCursor(Qt::WaitCursor));
    m_mainWin->findChild<GroupBoxBackInTime *>()->findChild<QPushButton *>("pushFetch")->setText("Cancel");
    m_mainWin->findChild<AttributeDataTreeView *>()->setDisabled(true);
    m_mainWin->findChild<QStatusBar *>()->showMessage("waiting for data from historical db...");
}

void UiHelper::extractFromHdbEnded()
{
    QApplication::restoreOverrideCursor();
    m_mainWin->findChild<QStatusBar *>()->showMessage("data from historical db fetched...");
    m_mainWin->findChild<GroupBoxBackInTime *>()->findChild<QPushButton *>("pushFetch")->setText("Fetch Data");
    m_mainWin->findChild<AttributeDataTreeView *>()->setDisabled(false);
}
